package il.ac.tau.cs.software1.mutli_collection;

import java.util.Collection;
import java.util.List;

public interface MultiCollection<T> extends Iterable<T> {

	/**
	 * Adds subCollection to the multi-collection
	 * 
	 * @pre subCollection != null
	 * @pre !subCollection.contains(null)
	 * @post getSubCollections().get(getSubCollections().size()-1) ==
	 *       subCollection)
	 */
	void addSubCollection(Collection<T> subCollection);

	/**
	 * Returns the list of added sub-collections
	 * 
	 * @post $ret != null
	 */
	List<Collection<T>> getSubCollections();

	/**
	 * Returns the value that appears in the multi-collection right before the
	 * first occurrence of the input value, according to the multi-collection's
	 * element order. If the input value does not appear in the order, returns
	 * null.
	 * 
	 * @pre value != null
	 */
	T getValueBeforeFirst(T value);

	/**
	 * Returns the value that appears in the multi-collection right after the
	 * last occurrence of the input value, according to the multi-collection's
	 * element order. If the input value does not appear in the order, returns
	 * null.
	 * 
	 * @pre value != null
	 */
	T getValueAfterLast(T value);

	/**
	 * Returns a new iterator that goes over the elements according to the
	 * multi-collection's order.
	 */
	MultiIterator<T> iterator();

}