package il.ac.tau.cs.software1.mutli_collection;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

public class SmallTestMultiCollections {

	private static final List<String> LIST0 = Arrays.asList("a11",
			"a12", "a13");
	private static final List<String> LIST1 = Arrays.asList("a21",
			"a22", "a23", "a24", "a25");
	private static final List<String> LIST2 = Arrays.asList("a31",
			"a32", "a33");
	private static final String[] expectedResults = new String[] {
			"a11(list 0)-a12(list 0)-a13(list 0)-a21(list 1)-a22(list 1)-a23(list 1)-a24(list 1)-a25(list 1)-a31(list 2)-a32(list 2)-a33(list 2)-",
			"a11(list 0)-a21(list 1)-a31(list 2)-a12(list 0)-a22(list 1)-a32(list 2)-a13(list 0)-a23(list 1)-a33(list 2)-a24(list 1)-a25(list 1)-",
			"a11(list 0)-a12(list 0)-a13(list 0)-a24(list 1)-a25(list 1)-" };
	private static final String[] expectedBeforeResults = new String[] {
			"a23", "a33", "a13" };
	private static final String[] expectedAfterResults = new String[] {
			"a12", "a21", "a12" };

	public static void main(String[] args) {
		List<MultiCollection<String>> multiCollections = createDifferentMultiCollections();

		for (int i = 0; i < multiCollections.size(); i++) {
			MultiCollection<String> multiCollection = multiCollections
					.get(i);
			System.out.println("*** Testing "
					+ multiCollection.getClass().getSimpleName());
			initMultiCollection(multiCollection);
			testOrder(multiCollection, expectedResults[i]);
			testBeforeFirst(multiCollection, expectedBeforeResults[i]);
			testAfterLast(multiCollection, expectedAfterResults[i]);
			System.out.println();
		}
	}

	private static List<MultiCollection<String>> createDifferentMultiCollections() {
		List<MultiCollection<String>> multiCollections = new LinkedList<>();
		multiCollections.add(new ConcatMultiCollection<String>());
		multiCollections.add(new RoundRobinMultiCollection<String>());
		multiCollections.add(new MajorityMultiCollection<String>());
		return multiCollections;
	}

	private static void testOrder(
			MultiCollection<String> multiCollection,
			String expectedResult) {
		StringBuilder builder = new StringBuilder();
		for (MultiIterator<String> iterator = multiCollection
				.iterator(); iterator.hasNext();) {
			int collectionIdx = iterator.getNextIdx();
			String s = iterator.next();
			builder.append(s + "(list " + collectionIdx + ")-");
		}
		String result = builder.toString();
		System.out.println("Result:\t\t" + result);
		System.out.println("Expected:\t" + expectedResult);
		assertTrue(expectedResult.equals(result));
	}

	private static void testBeforeFirst(
			MultiCollection<String> multiCollection,
			String expectedResult) {
		String before = multiCollection.getValueBeforeFirst("a24");
		System.out.println("Before first: got " + before
				+ ", expected " + expectedResult);
		assertTrue(expectedResult.equals(before));
	}

	private static void testAfterLast(
			MultiCollection<String> multiCollection,
			String expectedResult) {
		String after = multiCollection.getValueAfterLast("a11");
		System.out.println("After last: got " + after + ", expected "
				+ expectedResult);
		assertTrue(expectedResult.equals(after));
	}

	private static void initMultiCollection(
			MultiCollection<String> multiCollection) {
		multiCollection.addSubCollection(LIST0);
		multiCollection.addSubCollection(LIST1);
		multiCollection.addSubCollection(LIST2);
	}

	private static void assertTrue(boolean isSuccess) {
		System.out.println(">>> "
				+ (isSuccess ? "Success" : "Failure") + "!");
	}
}
